#!/bin/sh
# the next line restarts using wish -*- tcl -*- \
exec wish "$0" "$@"

# tksmacc: Small personal accounting program
#	under Tcl/Tk >= 7.6/Tk4.2.
#
# Copyright (C) 1998-2011 JF Moine
# Distributed under the GPL (GNU GENERAL PUBLIC LICENSE)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

set version "0.10.7"
#for tcl 7.6
set tcl_precision 17

#------------- options -------------
# may be changed in ~/.tksmaccrc (unix) or ~/tksmacc.rc (windows) ---
# 1 euro in local unit (here: french francs)
set euro_change 6.55957

# today and format to extract the month for such a date
set glovar(year) [clock format [clock seconds] -format {%Y}]
set glovar(get_month) {%02d.%02d}
set month(today) [expr {[clock format [clock seconds] -format {%N}] + 0}]
# format of total lines
set glovar(tlf) "%30s: %10.2f  %10.2fe\n"
set glovar(tl1) "%30s: %10.2f\n"

proc user_backup {fn} {
# backup procedure: you may copy the local file '$fn.acc' to elsewhere
#    puts stderr "no backup"
}

#set language english
set language francais
#------------- end options -------------

#---------- language specific section ----------
# language dependant texts
# (add and/or adapt to your country)
proc set_english {} {
#-- english --
    global lang version
# transaction header
#	  0....v....1....v....2....v....3....v....4....v....5....v....6....v....7....v..
#	  cccccccccc | llllllllllllllllllllllllllllll | dd/dd/dddd | sssssss.ssU | c
    array set lang {
	header
	"    check   |          description           |    date    |    value    |clean "
	new {(new)}
	help "Shortcuts:
- '1'..'9': select an account
- 'i' or '#': toggle the
    'ignore' flag
- 'p': purge (remove and save
    the clean transactions)
- 'q': quit
- 'r': research
- 's': save
- '/': search again
- <space>: toggle the 'clean'
    flag
- <return>: update or create
    a transaction"
	month {Month: }
	month_name { {} Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec}
	save {Save account %s?}
	old_acc {Old bank balance}
	new_acc {New bank balance}
	real_acc {real}
	move_title {Change}
	move {Change transaction}
	ok {OK}
	cancel {Dismiss}
	replace {Replace}
	add {Add}
	check {Check}
	desc {Description: }
	date {Date: }
	value {Value: }
	clean {Clean: }
	yes {Yes}
	no {No}
	notcorrect {'%s' is not a correct value}
	perso {Personnel account}
	menu_quit {Quit}
	menu_purge {Purge}
	menu_save {Save}
	menu_research {Research}
	menu_help {Help}
	chg {Account modified}
	next {Next}
	no_chg {No change}
	no_search {No previous search}
	saved {Account saved}
	not_found {not found}
    }
}

proc set_francais {} {
#-- francais --
    global lang version
# transaction header
#	  0....v....1....v....2....v....3....v....4....v....5....v....6....v....7....v..
#	  cccccccccc | llllllllllllllllllllllllllllll | dd/dd/dddd | sssssss.ssU | c
    array set lang {
	header
	"   chèque   |           libellé              |    date    |    somme    |tiré     "
	new {(nouveau)}
	help "Raccourcis:
- '1'..'9': selection
    d'un compte
- 'i' or '#': inverser
    l'indicateur 'ignorer'
- 'p': purge (supprime et sauve
    les opérations vues)
- 'q': quitter
- 'r': rechercher
- 's': sauver
- '/': rechercher la meme
    chaine
- <space>: inverser
    l'indicateur 'tiré'
- <return>: modifier ou créer
    une opération"
	month {Mois: }
	month_name { {} Jan Fev Mar Avr Mai Jun Jul Aou Sep Oct Nov Dec}
	save {Sauvegarde compte %s?}
	old_acc {Ancien compte banque}
	new_acc {Nouveau compte banque}
	real_acc {réel}
	move_title {Modification}
	move {Modification ligne du compte}
	ok {OK}
	cancel {Abandon}
	check {Chèque: }
	desc {Libellé: }
	date {Date: }
	value {Valeur: }
	clean {Tiré: }
	yes {Oui}
	no {Non}
	replace {Remplacer}
	add {Ajouter}
	notcorrect "'%s' n'est pas une valeur correcte"
	perso {Comptes maison}
	menu_quit {Quitter}
	menu_purge {Purge}
	menu_save {Sauvegarde}
	menu_research {Rechercher}
	menu_help {Aide}
	chg {Compte modifié}
	next {Suivant}
	no_chg {Compte non modifié}
	no_search {Pas de précédente recherche}
	not_found {non trouvé}
	saved {Compte sauvé}
    }
}
#---------- end language specific ----------

# get the configuration parameters
if {$tcl_platform(platform) == {unix}} {
    set rcfile "~/.tksmaccrc"
} else {
    set rcfile "~/tksmacc.rc"
}
if {[file exists $rcfile]} {
    source $rcfile
}

# initialize
    set_$language
    set upd_f 0
    set tk_strictMotif 1
    set two_currencies 0
#
    set glovar(lastdate) [format $glovar(get_month) \
		[clock format [clock seconds] -format {%d}] \
		$month(today)]
    
#
    if {$tcl_platform(platform) == {unix}} {
#	set font "-misc-fixed-bold-*-normal-*-14-*-*-*-*-*-*-*"
#	set font "Courier 10 bold"
	set font [list {Bitstream Vera Sans Mono} 10 bold]
#	set font "-misc-fixed-bold-*-normal-*-13-*-*-*-*-*-*-*"
#	set font "-*-lucidatypewriter-bold-r-*-*-13-*-*-*-*-*-iso8859-*"
	set font_ign [list {Bitstream Vera Sans Mono} 10]
    } else {
#	console show
	set font {Courier 10}
	set font_ign $font
    }
    set w [font measure $font { }]
# get current month and set a min/max month index table
    set month(cur) $month(today)
# balances
    array set balance {
	old_bank 0
	new_real 0
	new_bank 0
	old_be 0
	new_re 0
	new_be 0
    }
    array set search {
	index 0.0
	pattern {}
    }
# menubar
    frame .menuBar -relief raised -bd 2
    pack .menuBar -side top -fill x

    button .menuBar.quit -textvariable lang(menu_quit) \
	    -underline 0 -command quit -relief flat -padx 2
    button .menuBar.purge -textvariable lang(menu_purge) \
	    -underline 0 -command purgep -relief flat -padx 2
    button .menuBar.save -textvariable lang(menu_save) \
	    -underline 0 -command savef -relief flat -padx 2
    button .menuBar.research -textvariable lang(menu_research) \
	    -underline 0 -command research -relief flat -padx 2
    pack .menuBar.quit .menuBar.purge .menuBar.research .menuBar.save -side left

    button .menuBar.help -textvariable lang(menu_help) \
	    -underline 0 -command acc_help -relief flat -padx 2
    pack .menuBar.help -side right
# old and new balances
    text .total -height 3 -font $font -background AntiqueWhite
    pack .total -side top
# monthes
    frame .month -relief groove -bd 2
    label .month.l -text $lang(month)
    for {set i 1} {$i <= 12} {incr i} {
	button .month.m$i -text [lindex $lang(month_name) $i] \
		-command "month-change $i" -relief flat -padx 3
    }
    pack .month.l .month.m1 .month.m2 .month.m3 .month.m4 .month.m5 \
	     .month.m6 .month.m7 .month.m8 .month.m9 .month.m10 \
	     .month.m11 .month.m12 -side left -expand y -fill x
    pack .month -side top -fill x
    .month.m$month(today) configure -relief raised -background LightSkyBlue1
# transaction header
    label .entete -textvariable lang(header) -font $font
    pack .entete -side top
# frames per month with scrollbar containing the transactions
    set tabs "[expr $w * 13] [expr $w * 46] [expr $w * 64] numeric [expr $w * 73]"
    for {set i 1} {$i <= 12} {incr i} {
	frame .f$i
	text .f$i.list -yscroll ".f$i.scroll set" -width 77 \
		-font $font -height 22 \
		-background linen \
		-tabs $tabs
#	    -selectbackground LightSkyBlue1 -selectforeground black
	.f$i.list tag configure ope -background LightSkyBlue1
	.f$i.list tag configure clean -foreground SaddleBrown
# brown sienna SaddleBrown maroon chocolate4 brown4 khaki4
# LimeGreen
	.f$i.list tag configure ign -font $font_ign
	scrollbar .f$i.scroll -command ".f$i.list yview"
	pack .f$i.scroll -side right -fill y
	pack .f$i.list -side left -expand 1 -fill both
	bind .f$i.list <ButtonRelease-1> \
		"catch {.f$i.list tag remove ope ope.first ope.last}
		.f$i.list tag add ope \"@%x,%y linestart\" \"@%x,%y lineend\"
		break"
	bind .f$i.list <Double-1> {after idle invoke; break}
	.f$i.list insert end $lang(new) ope
    }
    pack .f$month(today) -side top -expand yes -fill y
# status which gets the keyboard events
    set status_line $lang(no_chg)
    label .status -textvariable status_line
    pack .status -side bottom
#    bind .status <KeyPress> {get_char %K; break}
    bind . <KeyPress> {get_char %K; break}
    bind . <ButtonRelease-4> {list-scroll -2; break}
    bind . <ButtonRelease-5> {list-scroll 2; break}

    bind Entry <Down> {focus [tk_focusNext %W]}
    bind Entry <Up> {focus [tk_focusPrev %W]}
    bind Button <Down> {focus [tk_focusNext %W]}
    bind Button <Up> {focus [tk_focusPrev %W]}
    bind Button <Return> {%W invoke}

    wm protocol . WM_DELETE_WINDOW quit
#    wm focusmodel . active

# -- compiled functions --
proc acc_help {} {
# help
    global version lang
    tk_messageBox -type ok \
	    -message "  tksmacc version $version
$lang(help)"
}

proc balance_add {line} {
# add a line to the balance
    global balance
    if {[string index $line 0] == {#}} return
    set toto [split $line "\t"]
    set hval [lindex $toto 3]
    set u {}
    scan $hval {%[-0123456789.]%[e]} val u
    if {[string compare $u e] == 0} {
	set balance(new_re) [expr $balance(new_re) + $val]
	if {[lindex $toto 4] != {n}} {
	    set balance(new_be) [expr $balance(new_be) + $val]
	}
    } else {
	set balance(new_real) [expr $balance(new_real) + $val]
	if {[lindex $toto 4] != {n}} {
	    set balance(new_bank) [expr $balance(new_bank) + $val]
	}
    }
}

proc balance_sub {line} {
# remove a line from the balance
    global balance
    if {[string index $line 0] == {#}} return
    set toto [split $line "\t"]
    set hval [lindex $toto 3]
    set u {}
    scan $hval {%[-0123456789.]%[e]} val u
    if {[string compare $u e] == 0} {
	set balance(new_re) [expr $balance(new_re) - $val]
	if {[lindex $toto 4] != {n}} {
	    set balance(new_be) [expr $balance(new_be) - $val]
	}
    } else {
	set balance(new_real) [expr $balance(new_real) - $val]
	if {[lindex $toto 4] != {n}} {
	    set balance(new_bank) [expr $balance(new_bank) - $val]
	}
    }
}

proc check_save {} {
# check if some transaction change and save on disk
    global fname upd_f lang
    if {$upd_f == 0} {return 0}
    set toto [tk_messageBox -type yesnocancel \
	    -message [format $lang(save) $fname]]
    switch $toto {
	yes {
	    savef
	    return 0
	}
	no {
	    set upd_f 0
	    return 0
	}
    }
    return -1
}

proc clean_toggle {} {
# toggle 'seen by the bank' or not
    global lang upd_f balance status_line month
    set ope [lindex [.f$month(cur).list tag ranges ope] 0]
    set line [.f$month(cur).list get "$ope linestart" "$ope lineend"]
    if {[string compare $line $lang(new)] == 0} return
    if {[string index $line 0] == {#}} return
    set toto [split $line "\t"]
    set hval [lindex $toto 3]
    set u { }
    scan $hval {%[-0123456789.]%[e]} val u
    if {[lindex $toto 4] == {n}} {
	regsub {n$} $line {y} line
	if {[string compare $u e] == 0} {
	    set balance(new_be) [expr $balance(new_be) + $val]
	} else {
	    set balance(new_bank) [expr $balance(new_bank) + $val]
	}
    } else {
	regsub {y$} $line {n} line
	if {[string compare $u e] == 0} {
	    set balance(new_be) [expr $balance(new_be) - $val]
	} else {
	    set balance(new_bank) [expr $balance(new_bank) - $val]
	}
    }
    .f$month(cur).list configure -state normal
    .f$month(cur).list delete "$ope linestart" "$ope lineend"
    .f$month(cur).list insert $ope $line ope
    if {[string index $line end] == {y}} {
	.f$month(cur).list tag add clean "$ope linestart" "$ope lineend"
    }
    .f$month(cur).list configure -state disabled
    set upd_f 1
    set status_line $lang(chg)
    dsp_total
}

proc def_accounts {} {
# define the accounts
    global nb_accounts
    set accounts [lsort [glob -nocomplain *.acc]]
    if {[llength $accounts] == 0} {
	set accounts noname
	set fd [open "$accounts.acc" w]
	puts $fd "acc $accounts\t0\t0"
	close $fd
    }
    set cnt 1
    foreach c $accounts {
	set n [file rootname $c]
	button .menuBar.$cnt -text "$cnt-$n" \
		-underline 0 -command [list loadf $n] -relief flat -padx 2
	pack .menuBar.$cnt -side left
	incr cnt
    }
    set nb_accounts $cnt
}

proc dsp_total {} {
# display the balances
    global balance euro_change lang glovar two_currencies

#puts [array names balance]
    .total configure -state normal
    .total delete 0.0 end
    if {$two_currencies} {
	.total insert end [format $glovar(tlf) \
	    $lang(old_acc) \
	    [expr $balance(old_bank) + $balance(old_be) * $euro_change] \
	    [expr $balance(old_be) + $balance(old_bank) / $euro_change]]
	.total insert end [format $glovar(tlf) \
	    $lang(new_acc) \
	    [expr $balance(new_bank) + $balance(new_be) * $euro_change] \
	    [expr $balance(new_be) + $balance(new_bank) / $euro_change]]
	.total insert end [format $glovar(tlf) \
	    $lang(real_acc) \
	    [expr $balance(new_real) + $balance(new_re) * $euro_change] \
	    [expr $balance(new_re) + $balance(new_real) / $euro_change]]
    } else {
	.total insert end [format $glovar(tl1) \
	    $lang(old_acc) $balance(old_bank)]
	.total insert end [format $glovar(tl1) \
	    $lang(new_acc) $balance(new_bank)]
	.total insert end [format $glovar(tl1) \
	    $lang(real_acc) $balance(new_real)]
    }
    .total configure -state disabled
}

proc exec-search {first} {
# search a string
    global lang search month
    if {$search(pattern) == {}} {
	tk_messageBox -type ok \
		-message $lang(no_search)
	return
    }
    if {$first} {
	set search_month [expr {$month(today) % 12 + 1}]
	set search_index 0.0
    } else {
	set search_month $month(cur)
	set search_index [lindex [.f$month(cur).list tag ranges ope] 0]
    }
    for {} {1} {} {
	if {[set ope [.f$search_month.list search -nocase -- \
		$search(pattern) $search_index+1l end-1l]] != {}} break
	if {$search_month == $month(today)} {
	    tk_messageBox -type ok \
		    -message "'$search(pattern)' $lang(not_found)"
	    return
	}
	set search_month [expr {$search_month % 12 + 1}]
	set search_index 0.0
    }
# pattern found: display the operation
    month-change $search_month
    .f$month(cur).list tag remove ope ope.first ope.last
    .f$month(cur).list tag add ope "$ope linestart" "$ope lineend"
    .f$month(cur).list see $ope
}

proc get_char {c} {
# treat a keyboard event
    global nb_accounts search month
    set delta 0
    switch $c {
	1 - 2 - 3 - 4 - 5 - 6 - 7 - 8 {
	    if {$c <= $nb_accounts} {
		.menuBar.$c invoke
	    }
	}
	H - h - question {acc_help}
	I - i - numbersign {ignore_toggle}
	P - p purgep
	Q - q quit
	R - r research
	S - s savef
	slash {
	    exec-search 0
	}
	space clean_toggle
	Down - KP_Down {set delta 1}
	KP_Begin {month-change $month(today)}
	KP_Left - Left {
	    set new_month [expr {($month(cur) + 10) % 12 + 1}]
	    month-change $new_month
	}
	KP_Right - Right {
	    set new_month [expr {$month(cur) % 12 + 1}]
	    month-change $new_month
	}
	Up - KP_Up {set delta -1}
	Next -
	KP_Next {set delta 2}
	Prior -
	KP_Prior {set delta -2}
	Return -
	KP_Enter {after idle invoke}
	default {
#	    puts stderr "c:$c"
	}
    }
    if {$delta != 0} {
	list-scroll $delta
    }
}

proc ignore_toggle {} {
# toggle 'ignore' or not
    global lang upd_f status_line month
    set ope [lindex [.f$month(cur).list tag ranges ope] 0]
    set line [.f$month(cur).list get "$ope linestart" "$ope lineend"]
    if {$line == $lang(new)} return
    .f$month(cur).list configure -state normal
    if {[string index $line 0] == {#}} {
	.f$month(cur).list delete "$ope linestart" "$ope linestart +1 chars"
	.f$month(cur).list tag remove ign "$ope linestart" "$ope lineend"
	set line [string range $line 1 end]
	balance_add $line
    } else {
	balance_sub $line
	.f$month(cur).list insert "$ope linestart" "#" ope
	.f$month(cur).list tag add ign "$ope linestart" "$ope lineend"
    }
    .f$month(cur).list configure -state disabled
    set upd_f 1
    set status_line $lang(chg)
    dsp_total
}

proc invoke {} {
# get a transaction
    global fname clean fname glovar lang month
    set ope [lindex [.f$month(cur).list tag ranges ope] 0]
    set line [.f$month(cur).list get "$ope linestart" "$ope lineend"]
    catch {destroy .invoke}
    toplevel .invoke
    wm title .invoke "$lang(move_title) $fname"
    wm iconname .invoke {change}
#    wm transient .invoke [winfo toplevel [winfo parent .invoke]]
    set x [winfo rootx .]
    set y [winfo rooty .]
    incr x 250
    incr y 100
    wm geometry .invoke +$x+$y

    label .invoke.msg -text "$lang(move) $fname"

    frame .invoke.cheque -bd 2
    entry .invoke.cheque.entry -relief sunken -width 10 -background AntiqueWhite
    label .invoke.cheque.text -width 10 -anchor e -textvariable lang(check)
    pack .invoke.cheque.text .invoke.cheque.entry -side left

    frame .invoke.lib -bd 2
    entry .invoke.lib.entry -relief sunken -width 30 -background AntiqueWhite
    label .invoke.lib.text -width 10 -anchor e -textvariable lang(desc)
    pack .invoke.lib.text .invoke.lib.entry -side left

    frame .invoke.date -bd 2
    entry .invoke.date.entry -relief sunken -width 10 -background AntiqueWhite
    label .invoke.date.text -width 10 -anchor e -textvariable lang(date)
    pack .invoke.date.text .invoke.date.entry -side left

    frame .invoke.val -bd 2
    entry .invoke.val.entry -relief sunken -width 10 -background AntiqueWhite
    label .invoke.val.text -width 10 -anchor e -textvariable lang(value)
    pack .invoke.val.text .invoke.val.entry -side left

    frame .invoke.clean -bd 2
    label .invoke.clean.text -width 10 -anchor e -textvariable lang(clean)
    button .invoke.clean.b -text $lang(no) -relief sunken -pady 2 \
	-background AntiqueWhite -command invoke-clean
    pack .invoke.clean.text .invoke.clean.b -side left

    pack .invoke.msg .invoke.cheque .invoke.lib .invoke.date .invoke.val .invoke.clean \
	    -side top -fill x

    frame .invoke.buttons
    pack .invoke.buttons -side top -fill x -pady 2m
    button .invoke.buttons.ok -text $lang(ok) -command {invoke_ok}
    button .invoke.buttons.cancel -text $lang(cancel) -command {destroy .invoke}
    pack .invoke.buttons.ok .invoke.buttons.cancel -side left -expand 1

    set toto [split $line "\t"]
    set cheque [lindex $toto 0]
    if {$cheque != $lang(new)} {
	set lib [lindex $toto 1]
	set date [lindex $toto 2]
	set val [lindex $toto 3]
	set clean [lindex $toto 4]
	if {[string compare $clean y] == 0} {
	   .invoke.clean.b configure -text $lang(yes)
	}
	.invoke.cheque.entry insert 0 $cheque
	.invoke.lib.entry insert 0 $lib
	.invoke.date.entry insert 0 $date
	.invoke.val.entry insert 0 $val
	button .invoke.buttons.new -textvariable lang(add) \
		-command { .f$month(cur).list tag remove ope ope.first ope.last
	set ope [.f$month(cur).list index {end -1 lines}]
	.f$month(cur).list tag add ope "$ope linestart" "$ope lineend"
	.f$month(cur).list see $ope
    }
	.invoke.buttons.ok configure -textvariable lang(replace)
	pack .invoke.buttons.new -side left -expand 1
    } else {
	.invoke.date.entry insert 0 $glovar(lastdate)
	set clean {n}
    }

    focus .invoke.cheque.entry
    tkwait visibility .invoke
    grab .invoke
#    tkwait window .invoke
#    focus  .status
}

proc invoke-clean {} {
# change the clean flag
    global clean lang
    if {[string compare $clean n] == 0} {
	set clean {y}
	set t $lang(yes)
    } else {
	set clean {n}
	set t $lang(no)
    }
   .invoke.clean.b configure -text $t
}

proc invoke_ok {} {
# ok for the transaction
    global balance upd_f clean glovar lang status_line month two_currencies
    set ope [lindex [.f$month(cur).list tag ranges ope] 0]
    set hval [.invoke.val.entry get]
    set val 0
    set u {}
    scan $hval {%[+-0123456789.,]%[Ee]} val u
    regsub "," $val "." val
    if {[catch {expr 0.0 + $val} val]} {
	tk_messageBox -type ok -icon error \
		-message [format $lang(notcorrect) $hval]
	return
    }
    set val [format {%.2f} $val]
#    if {$u == "E"} {
#	set u "e"
#    }
    if {$two_currencies} {
# (force euro)
	set u {e}
    }
    .f$month(cur).list tag remove ope ope.first ope.last
    .f$month(cur).list configure -state normal
    set line [.f$month(cur).list get "$ope linestart" "$ope lineend"]
    if {$line != $lang(new)} {
	.f$month(cur).list delete "$ope linestart" "$ope lineend +1 chars"
	balance_sub $line
    } else {
	set ope [.f$month(cur).list index {end -1 lines}]
    }

    set glovar(lastdate) [.invoke.date.entry get]
    set line "[.invoke.cheque.entry get]\t[.invoke.lib.entry get]\t$glovar(lastdate)\t$val$u\t$clean"
    .f$month(cur).list insert $ope "$line\n"
    if {[string index $line 0] == {#}} {
	.f$month(cur).list tag add ign "$ope linestart" "$ope lineend"
    }
    if {[string index $line end] == {y}} {
	.f$month(cur).list tag add clean "$ope linestart" "$ope lineend"
    }
    balance_add $line
    .f$month(cur).list configure -state disabled
    set upd_f 1
    set status_line $lang(chg)
    destroy .invoke
    dsp_total
    set ope [.f$month(cur).list index "$ope +1 lines"]
    .f$month(cur).list tag add ope "$ope linestart" "$ope lineend"
    .f$month(cur).list see $ope
}

proc loadf {fn} {
# load an account
    global balance fname glovar lang month two_currencies
    if {[check_save]} return
# remove all lines but the 'new' one
    for {set m 1} {$m <= 12} {incr m} {
	.f$m.list configure -state normal
	.f$m.list delete 0.0 end-1lines
    }
# get the bank balance
    set fname $fn
    set fd [open "$fname.acc" r]
    gets $fd line
    set toto [split $line "\t"]
    if {[llength $toto] == 2} {
	set balance(old_bank) [lindex $toto 1]
	set balance(old_be) $balance(old_bank)
	set two_currencies 0
    } else {
	set balance(old_bank) [lindex $toto 1]
	set balance(old_be) [lindex $toto 2]
	set two_currencies 1
    }
    set balance(new_real) $balance(old_bank)
    set balance(new_bank) $balance(old_bank)
    set balance(new_re) $balance(old_be)
    set balance(new_be) $balance(old_be)
# scan the operations and put them in the right month window
    set m $month(today)
    while {[gets $fd line] >= 0} {
	if {[string length $line] == 0} continue
	set toto [split $line "\t"]
	catch {scan [lindex $toto 2] $glovar(get_month) d m}
	if {$m < 1 || $m > 12} {
	    set m $month(today)
	}
	if {[string index $line 0] == {#}} {
	    .f$m.list insert end-1lines "$line\n" ign
	} elseif {[string index $line end] == {y}} {
	    .f$m.list insert end-1lines "$line\n" clean
	} else {
	    .f$m.list insert end-1lines "$line\n"
	}
	balance_add $line
    }
    close $fd
# put some colors on the month bar and select the 'new' line
    for {set m 1} {$m <= 12} {incr m} {
	if {[.f$m.list index end] > 2.0 && $m != $month(today)} {
	    .month.m$m configure -background LightGreen
	}
	catch {.f$m.list tag remove ope ope.first ope.last}
	set ope [.f$m.list index end-1lines]
	.f$m.list tag add ope "$ope linestart" "$ope lineend"
	.f$m.list see $ope
	.f$m.list configure -state disabled
    }
    wm title . "$lang(perso) - $fname"
    month-change $month(today)
#    .f$month(cur).list see end
#    .f$month(cur).list configure -state disabled
    dsp_total
}

proc list-scroll {delta} {
	global month
	set w .f$month(cur).list
	set ope0 [lindex [$w tag ranges ope] 0]
	$w tag remove ope ope.first ope.last
	switch $delta {
	    2 {
		set delta [expr {[.f$month(cur).list cget -height] - 2}]
	    }
	    -2 {
		set delta [expr {2 - [.f$month(cur).list cget -height]}]
	    }
	}
	if {$delta > 0} {
	    set d +$delta
	} else {
	    set d $delta
	}
	set ope [$w index "$ope0 $d lines"]
	if {$ope >= [$w index end]} {
	    set ope [$w index {end-1l}]
	}
	$w tag add ope "$ope linestart" "$ope lineend"
	if {$delta > 0} {
	    if {$delta == 1} {
		$w see "$ope+6lines"
	    } else {
		$w yview scroll $delta units
	    }
	} else {
	    if {$delta == -1} {
		$w see "$ope-6lines"
	    } else {
		$w yview scroll $delta units
	    }
	}
}

proc month-change {new_month} {
# display a new month
    global month
    pack forget .f$month(cur)
    .month.m$month(cur) configure -relief flat
    set month(cur) $new_month
    pack .f$month(cur) -side top -expand yes -fill y
    .month.m$month(cur) configure -relief raised
}

proc purgep {} {
# save and remove the transactions seen by the bank
    global balance fname glovar lang status_line
    global upd_f month
    set fd {}
    for {set m 1} {$m <= 12} {incr m} {
	.f$m.list configure -state normal
	set i 0.0
	while {1} {
	    set line [.f$m.list get $i "$i lineend"]
	    if {[string compare $line $lang(new)] == 0} break
	    if {[string index $line 0] != "#" \
		    && [string index $line [expr [string length $line] - 1]] == "y"} {
		if {$fd == {}} {
		    set suf $glovar(year)
		    if {$m > $month(today)} {
			incr suf -1
		    }
		    set fd [open "$fname.acc.$suf" a+]
		    puts $fd "### [clock format [clock seconds] -format {purge %Y/%m/%d}] ###"
		}
		puts $fd $line
		set toto [split $line "\t"]
		set hval [lindex $toto 3]
		set u { }
		scan $hval {%[-0123456789.]%[Ee]} val u
		if {$u == "e"} {
		    set balance(old_be) [expr $balance(old_be) + $val]
		} else {
		    set balance(old_bank) [expr $balance(old_bank) + $val]
		}
		.f$m.list delete $i "$i lineend +1chars"
	    } else {
		set i [.f$m.list index "$i +1lines"]
	    }
	}
	if {[.f$m.list tag ranges ope] == {}} {
	    set ope [.f$m.list index {end-1lines}]
	    .f$m.list tag add ope "$ope linestart" "$ope lineend"
	}
	.f$m.list configure -state disabled
	if {$m == $month(today) && $fd != {}} {
	    set upd_f 1
	    close $fd
	    set fd {}
	}
    }
    if {$fd != {}} {
	set upd_f 1
	close $fd
    }
    if {$upd_f} {
	set status_line $lang(chg)
    }
# if no selection, access to the 'new' line
    if {[.f$month(cur).list tag ranges ope] == {}} {
	set ope [.f$month(cur).list index {end-1lines}]
	.f$month(cur).list tag add ope "$ope linestart" "$ope lineend"
	.f$month(cur).list see $ope
    }
# redisplay the balance
    dsp_total
}

proc quit {} {
# exit program
    if {[check_save]} return
    exit 0
}

proc research {} {
# get the chain to search
    global lang month search
    catch {destroy .seach}
    toplevel .search
    wm title .search "$lang(menu_research)"
    wm iconname .search {change}
    frame .search.f -bd 2
    label .search.f.text -width 10 -anchor e -textvariable lang(menu_research)
    entry .search.f.entry -relief sunken -width 10 -background AntiqueWhite \
	    -textvariable search(pattern)
    pack .search.f.text .search.f.entry -side left
    frame .search.buttons
    set search(button) {}
    button .search.buttons.ok -textvariable lang(ok) \
	    -command {set search(button) 1; destroy .search}
    button .search.buttons.next -textvariable lang(next) \
	    -command {set search(button) 0; destroy .search}
    button .search.buttons.cancel -textvariable lang(cancel) \
	    -command {destroy .search}
    pack .search.buttons.ok .search.buttons.ok .search.buttons.cancel -side left -expand 1
    pack .search.f .search.buttons -side top
    .search.f.entry selection range 0 end
    bind .search.f.entry <Return> {.search.buttons.ok invoke; break}
    bind .search.buttons.ok <Return> {.search.buttons.ok invoke}
    bind .search.buttons.cancel <Return> {destroy .search}
    grab .search
    focus .search.f.entry
    tkwait window .search
    if {$search(button) != {}} {exec-search $search(button)}
}

proc savef {} {
# save the account on disk
    global balance status_line two_currencies
    global fname upd_f lang month
    if {!$upd_f} return
    if {[file exists $fname.acc]} {
	file rename -force $fname.acc $fname.acc~
    }
    set fd [open "$fname.acc" w]
#    puts $fd "acc $fname compte banque\t$balance(old_bank)\t$balance(old_be)"
    if {$two_currencies} {
	puts $fd [format "acc %s\t%.2f\t%.2f" \
	    $fname $balance(old_bank) $balance(old_be)]
    } else {
	puts $fd [format "acc %s\t%.2f" $fname $balance(old_bank)]
    }
    for {set i 1} {$i <= 12} {incr i} {
# suppose there is nothing after 5 monthes
	set j [expr {($i + $month(today) + 4) % 12 + 1}]
	if {[.f$j.list index end] > 2.0} {
	    puts $fd [.f$j.list get 0.0 [.f$j.list index end-1l-1c]]
	}
    }
    close $fd
    user_backup $fname
    set upd_f 0
    set status_line $lang(saved)
}

##### main #####
def_accounts

.menuBar.1 invoke

#focus .status
